package net.sf.seaf.persistence.hibernate;

import java.io.Serializable;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Properties;

import org.hibernate.Hibernate;
import org.hibernate.HibernateException;
import org.hibernate.usertype.EnhancedUserType;
import org.hibernate.usertype.ParameterizedType;
import org.hibernate.util.ReflectHelper;

@SuppressWarnings({ "rawtypes", "unchecked" })
public class StringEnumUserType implements EnhancedUserType, ParameterizedType {

	private static final String PARAM_CLASS_NAME = "className";

	private Class<Enum> clazz;

	public void setParameterValues(Properties parameters) {
		String className = parameters.getProperty(PARAM_CLASS_NAME);
		try {
			clazz = ReflectHelper.classForName(className);
		} catch (ClassNotFoundException e) {
			throw new HibernateException("Enum class not found: " + className,
					e);
		}
	}

	public Class<Enum> returnedClass() {
		return clazz;
	}

	public int[] sqlTypes() {
		return new int[] { Hibernate.STRING.sqlType() };
	}

	public Object nullSafeGet(ResultSet rs, String[] names, Object owner)
			throws HibernateException, SQLException {
		String name = rs.getString(names[0]);
		return rs.wasNull() ? null : Enum.valueOf(clazz, name);
	}

	public void nullSafeSet(PreparedStatement st, Object value, int index)
			throws HibernateException, SQLException {
		if (value == null) {
			st.setNull(index, Hibernate.STRING.sqlType());
		} else {
			st.setString(index, ((Enum) value).name());
		}
	}

	public Object fromXMLString(String xmlValue) {
		return Enum.valueOf(clazz, xmlValue);
	}

	public String toXMLString(Object value) {
		return ((Enum) value).name();
	}

	public String objectToSQLString(Object value) {
		return '\'' + ((Enum) value).name() + '\'';
	}

	public Object deepCopy(Object value) throws HibernateException {
		return value;
	}

	public boolean isMutable() {
		return false;
	}

	public Object assemble(Serializable cached, Object owner)
			throws HibernateException {
		return cached;
	}

	public Serializable disassemble(Object value) throws HibernateException {
		return (Serializable) value;
	}

	public Object replace(Object original, Object target, Object owner)
			throws HibernateException {
		return original;
	}

	public int hashCode(Object x) throws HibernateException {
		return x.hashCode();
	}

	public boolean equals(Object x, Object y) throws HibernateException {
		if (x == y)
			return true;
		if (null == x || null == y)
			return false;
		return x.equals(y);
	}

}
